@@ -24,7 +24,7 @@
  * @author Samuel Mimram, David Baelde
  */
 
-/* $Id: mad_stubs.c 4672 2007-10-17 16:48:18Z metamorph68 $ */
+/* $Id: mad_stubs.c 5355 2008-04-02 09:15:13Z metamorph68 $ */
 
 #include <caml/alloc.h>
 #include <caml/callback.h>
@@ -390,9 +390,8 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
   int output_pos = 0;
   int i;
 
-  mf_fill_buffer(mf);
-  if(mf_decode(mf))
-    ocaml_mad_decode_frame(madf);
+  do { mf_fill_buffer(mf); }
+    while (mf_decode(mf) == 1);
 
   /* Synthesized samples must be converted from mad's fixed
    * point number to the consumer format. Here we use unsigned
@@ -434,9 +433,8 @@ CAMLprim value ocaml_mad_decode_frame_float(value madf)
   int chans;
   int i, c;
 
-  mf_fill_buffer(mf);
-  if(mf_decode(mf))
-    ocaml_mad_decode_frame_float(madf);
+  do { mf_fill_buffer(mf); }
+    while (mf_decode(mf) == 1);
 
   chans = MAD_NCHANNELS(&mf->frame.header);
   ret = caml_alloc_tuple(chans);
